@@ -1,16 +1,7 @@
 """
-
-prompt_toolkit
---------------
-
-Pure Python alternative to readline.
-
-Still experimental and incomplete. It should be able to handle RAW vt100 input
-sequences for a command line and construct a command line with autocompletion
-there.
+prompt_toolkit: Pure Python alternative to readline.
 
 Author: Jonathan Slenders
-
 """
 from __future__ import unicode_literals
 
@@ -18,6 +9,7 @@ import errno
 import fcntl
 import os
 import select
+import signal
 import six
 import sys
 import threading
@@ -141,7 +133,7 @@ class CommandLineInterface(object):
         if create_async_autocompleters:
             for n, l in self.lines.items():
                 if l.completer:
-                    l.onTextInsert += self._create_async_complter(n)
+                    l.onTextInsert += self._create_async_completer(n)
 
         self._reset()
 
@@ -265,6 +257,21 @@ class CommandLineInterface(object):
             self._stdin_decoder = self.stdin_decoder_cls()
             return ''
 
+    def _on_resize(self):
+        """
+        When the window size changes, we erase the current output and request
+        again the cursor position. We the CPR answer arrives, the output is
+        drawn again.
+        (We do it asynchronously, because writing to the output from inside the
+        signal handler causes easily reentrant calls, giving runtime errors.)
+        """
+        assert self.is_reading_input
+
+        def do_in_event_loop():
+            self.renderer.erase()
+            self.renderer.request_absolute_cursor_position()
+        self.call_from_executor(do_in_event_loop)
+
     def read_input(self, initial_value='', on_abort=AbortAction.RETRY, on_exit=AbortAction.IGNORE):
         """
         Read input string from command line.
@@ -309,11 +316,7 @@ class CommandLineInterface(object):
 
                 self._redraw()
 
-                # When the window size changes, we do a redraw request call.
-                # (We do it asynchronously, because doing the actual drawing
-                # inside the signal handler causes easily reentrant calls,
-                # giving runtime errors.)
-                with call_on_sigwinch(self.request_redraw):
+                with call_on_sigwinch(self._on_resize):
                     while True:
                         c = self._get_char_loop()
 
@@ -388,6 +391,8 @@ class CommandLineInterface(object):
         (Not thread safe -- to be called from inside the key bindings.)
         Run system command.
         """
+        assert self.is_reading_input, 'Should be called while reading input.'
+
         self.renderer.erase()
 
         # Run system command.
@@ -398,6 +403,24 @@ class CommandLineInterface(object):
         self.renderer.reset()
         self.renderer.request_absolute_cursor_position()
 
+    def suspend_to_background(self):
+        """
+        (Not thread safe -- to be called from inside the key bindings.)
+        Suspend process.
+        """
+        assert self.is_reading_input, 'Should be called while reading input.'
+
+        self.renderer.erase()
+
+        # Make sure to be in cooked mode when suspending.
+        with cooked_mode(self.stdin.fileno()):
+            # Send `SIGSTP` to own process.
+            # This will cause it to suspend.
+            os.kill(os.getpid(), signal.SIGTSTP)
+
+        self.renderer.reset()
+        self.renderer.request_absolute_cursor_position()
+
     @property
     def is_exiting(self):
         return self._exit_flag
@@ -410,13 +433,13 @@ class CommandLineInterface(object):
     def is_returning(self):
         return self._return_code
 
-    def _create_async_complter(self, line_name):
+    def _create_async_completer(self, line_name):
         """
         Create function for asynchronous autocompletion while typing.
         (Autocomplete in other thread.)
         """
         line = self.lines[line_name]
-        complete_thread_running = [False] # By ref.
+        complete_thread_running = [False]  # By ref.
 
         def async_completer():
             document = line.document
